@@ -1,6 +1,8 @@
-import { V1ObjectMeta } from '@kubernetes/client-node'
+/* Copyright Contributors to the Open Cluster Management project */
+
+import { V1ObjectMeta } from '@kubernetes/client-node/dist/gen/model/v1ObjectMeta'
 import { listClusterResources } from '../lib/resource-request'
-import { IResource } from './resource'
+import { IResource, IResourceDefinition } from './resource'
 
 export const ClusterManagementAddOnApiVersion = 'addon.open-cluster-management.io/v1alpha1'
 export type ClusterManagementAddOnApiVersionType = 'addon.open-cluster-management.io/v1alpha1'
@@ -8,6 +10,11 @@ export type ClusterManagementAddOnApiVersionType = 'addon.open-cluster-managemen
 export const ClusterManagementAddOnKind = 'ClusterManagementAddOn'
 export type ClusterManagementAddOnKindType = 'ClusterManagementAddOn'
 
+export const ClusterManagementAddOnDefinition: IResourceDefinition = {
+    apiVersion: ClusterManagementAddOnApiVersion,
+    kind: ClusterManagementAddOnKind,
+}
+
 export interface ClusterManagementAddOn extends IResource {
     apiVersion: ClusterManagementAddOnApiVersionType
     kind: ClusterManagementAddOnKindType
