@@ -1,4 +1,6 @@
-import { V1Status } from '@kubernetes/client-node'
+/* Copyright Contributors to the Open Cluster Management project */
+
+import { V1Status } from '@kubernetes/client-node/dist/gen/model/v1Status'
 
 export const StatusApiVersion = 'v1'
 export type StatusApiVersionType = 'v1'
