@@ -1,5 +1,5 @@
 /* Copyright Contributors to the Open Cluster Management project */
-import { AcmAlert } from '@open-cluster-management/ui-components'
+import { AcmAlert } from '@stolostron/ui-components'
 import { useTranslation } from 'react-i18next'
 import { Cluster, clusterDangerStatuses } from '../../../../../lib/get-cluster'
 
