@@ -9,9 +9,9 @@ const url = require('url');
 
 const { ParseError } = require('got');
 const jose = require('jose');
-const base64url = require('base64url');
 const tokenHash = require('oidc-token-hash');
 
+const base64url = require('./helpers/base64url');
 const defaults = require('./helpers/defaults');
 const { assertSigningAlgValuesSupport, assertIssuerConfiguration } = require('./helpers/assert');
 const pick = require('./helpers/pick');
@@ -125,6 +125,17 @@ async function claimJWT(label, jwt) {
 }
 
 function getKeystore(jwks) {
+  if (!isPlainObject(jwks) || !Array.isArray(jwks.keys) || jwks.keys.some((k) => !isPlainObject(k) || !('kty' in k))) {
+    throw new TypeError('jwks must be a JSON Web Key Set formatted object');
+  }
+
+  // eslint-disable-next-line no-restricted-syntax
+  for (const jwk of jwks.keys) {
+    if (jwk.kid === undefined) {
+      jwk.kid = `DONOTUSE.${random()}`;
+    }
+  }
+
   const keystore = jose.JWKS.asKeyStore(jwks);
   if (keystore.all().some((key) => key.type !== 'private')) {
     throw new TypeError('jwks must only contain private keys');
@@ -724,6 +735,8 @@ module.exports = (issuer, aadIssValidation = false) => class Client extends Base
       });
     }
 
+    const fapi = this.constructor.name === 'FAPIClient';
+
     if (returnedBy === 'authorization') {
       if (!payload.at_hash && tokenSet.access_token) {
         throw new RPError({
@@ -739,19 +752,7 @@ module.exports = (issuer, aadIssValidation = false) => class Client extends Base
         });
       }
 
-      const fapi = this.constructor.name === 'FAPIClient';
-
       if (fapi) {
-        if (payload.iat < timestamp - 3600) {
-          throw new RPError({
-            printf: ['JWT issued too far in the past, now %i, iat %i', timestamp, payload.iat],
-            now: timestamp,
-            tolerance: this[CLOCK_TOLERANCE],
-            iat: payload.iat,
-            jwt: idToken,
-          });
-        }
-
         if (!payload.s_hash && (tokenSet.state || state)) {
           throw new RPError({
             message: 'missing required property s_hash',
@@ -773,6 +774,16 @@ module.exports = (issuer, aadIssValidation = false) => class Client extends Base
       }
     }
 
+    if (fapi && payload.iat < timestamp - 3600) {
+      throw new RPError({
+        printf: ['JWT issued too far in the past, now %i, iat %i', timestamp, payload.iat],
+        now: timestamp,
+        tolerance: this[CLOCK_TOLERANCE],
+        iat: payload.iat,
+        jwt: idToken,
+      });
+    }
+
     if (tokenSet.access_token && payload.at_hash !== undefined) {
       try {
         tokenHash.validate({ claim: 'at_hash', source: 'access_token' }, payload.at_hash, tokenSet.access_token, header.alg, key && key.crv);
@@ -1013,8 +1024,9 @@ module.exports = (issuer, aadIssValidation = false) => class Client extends Base
       method,
       headers,
       body,
-      tokenType = accessToken instanceof TokenSet ? accessToken.token_type : 'Bearer',
       DPoP,
+      // eslint-disable-next-line no-nested-ternary
+      tokenType = DPoP ? 'DPoP' : accessToken instanceof TokenSet ? accessToken.token_type : 'Bearer',
     } = {},
   ) {
     if (accessToken instanceof TokenSet) {
@@ -1039,7 +1051,7 @@ module.exports = (issuer, aadIssValidation = false) => class Client extends Base
       responseType: 'buffer',
       method,
       url: resourceUrl,
-    }, { mTLS, DPoP });
+    }, { accessToken, mTLS, DPoP });
   }
 
   /**
@@ -1425,6 +1437,12 @@ module.exports = (issuer, aadIssValidation = false) => class Client extends Base
     if (jwks !== undefined && !(metadata.jwks || metadata.jwks_uri)) {
       const keystore = getKeystore.call(this, jwks);
       metadata.jwks = keystore.toJWKS(false);
+      // eslint-disable-next-line no-restricted-syntax
+      for (const jwk of metadata.jwks.keys) {
+        if (jwk.kid.startsWith('DONOTUSE.')) {
+          delete jwk.kid;
+        }
+      }
     }
 
     const response = await request.call(this, {
@@ -1487,14 +1505,17 @@ module.exports = (issuer, aadIssValidation = false) => class Client extends Base
     let signed;
     let key;
 
+    const fapi = this.constructor.name === 'FAPIClient';
+    const unix = now();
     const header = { alg: signingAlgorithm, typ: 'oauth-authz-req+jwt' };
     const payload = JSON.stringify(defaults({}, requestObject, {
       iss: this.client_id,
       aud: this.issuer.issuer,
       client_id: this.client_id,
       jti: random(),
-      iat: now(),
-      exp: now() + 300,
+      iat: unix,
+      exp: unix + 300,
+      ...(fapi ? { nbf: unix } : undefined),
     }));
 
     if (signingAlgorithm === 'none') {
@@ -1521,7 +1542,7 @@ module.exports = (issuer, aadIssValidation = false) => class Client extends Base
 
       signed = jose.JWS.sign(payload, key, {
         ...header,
-        kid: symmetric ? undefined : key.kid,
+        kid: symmetric || key.kid.startsWith('DONOTUSE.') ? undefined : key.kid,
       });
     }
 
@@ -1547,6 +1568,57 @@ module.exports = (issuer, aadIssValidation = false) => class Client extends Base
     });
   }
 
+  /**
+   * @name pushedAuthorizationRequest
+   * @api public
+   */
+  async pushedAuthorizationRequest(params = {}, { clientAssertionPayload } = {}) {
+    assertIssuerConfiguration(this.issuer, 'pushed_authorization_request_endpoint');
+
+    const body = {
+      ...('request' in params ? params : authorizationParams.call(this, params)),
+      client_id: this.client_id,
+    };
+
+    const response = await authenticatedPost.call(
+      this,
+      'pushed_authorization_request',
+      {
+        responseType: 'json',
+        form: body,
+      },
+      { clientAssertionPayload, endpointAuthMethod: 'token' },
+    );
+    const responseBody = processResponse(response, { statusCode: 201 });
+
+    if (!('expires_in' in responseBody)) {
+      throw new RPError({
+        message: 'expected expires_in in Pushed Authorization Successful Response',
+        response,
+      });
+    }
+    if (typeof responseBody.expires_in !== 'number') {
+      throw new RPError({
+        message: 'invalid expires_in value in Pushed Authorization Successful Response',
+        response,
+      });
+    }
+    if (!('request_uri' in responseBody)) {
+      throw new RPError({
+        message: 'expected request_uri in Pushed Authorization Successful Response',
+        response,
+      });
+    }
+    if (typeof responseBody.request_uri !== 'string') {
+      throw new RPError({
+        message: 'invalid request_uri value in Pushed Authorization Successful Response',
+        response,
+      });
+    }
+
+    return responseBody;
+  }
+
   /**
    * @name issuer
    * @api public
@@ -1605,7 +1677,7 @@ Object.defineProperty(BaseClient.prototype, 'validateJARM', {
  * @name dpopProof
  * @api private
  */
-function dpopProof(payload, jwk) {
+function dpopProof(payload, jwk, accessToken) {
   if (!isPlainObject(payload)) {
     throw new TypeError('payload must be a plain object');
   }
@@ -1632,6 +1704,7 @@ function dpopProof(payload, jwk) {
   return jose.JWS.sign({
     iat: now(),
     jti: random(),
+    ath: accessToken ? base64url.encode(crypto.createHash('sha256').update(accessToken).digest()) : undefined,
     ...payload,
   }, jwk, {
     alg,
@@ -1645,7 +1718,7 @@ Object.defineProperty(BaseClient.prototype, 'dpopProof', {
   configurable: true,
   value(...args) {
     process.emitWarning(
-      'The DPoP APIs implements an IETF draft. Breaking draft implementations are included as minor versions of the openid-client library, therefore, the ~ semver operator should be used and close attention be payed to library changelog as well as the drafts themselves.',
+      'The DPoP APIs implements an IETF draft (https://www.ietf.org/archive/id/draft-ietf-oauth-dpop-03.html). Breaking draft implementations are included as minor versions of the openid-client library, therefore, the ~ semver operator should be used and close attention be payed to library changelog as well as the drafts themselves.',
       'DraftWarning',
     );
     Object.defineProperty(BaseClient.prototype, 'dpopProof', {
